@@ -30,6 +30,8 @@
 
 #include "sshincomingpacket_p.h"
 
+#include "ssh_global.h"
+#include "sshbotanconversions_p.h"
 #include "sshcapabilities_p.h"
 
 namespace QSsh {
@@ -161,49 +163,86 @@ SshKeyExchangeInit SshIncomingPacket::extractKeyExchangeInitData() const
             = SshPacketParser::asNameList(m_data, &offset);
         exchangeData.firstKexPacketFollows
             = SshPacketParser::asBool(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_KEY_EXCHANGE_FAILED,
             "Key exchange failed: Server sent invalid SSH_MSG_KEXINIT packet.");
     }
     return exchangeData;
 }
 
-SshKeyExchangeReply SshIncomingPacket::extractKeyExchangeReply(const QByteArray &pubKeyAlgo) const
+static void getHostKeySpecificReplyData(SshKeyExchangeReply &replyData,
+                                        const QByteArray &hostKeyAlgo, const QByteArray &input)
+{
+    quint32 offset = 0;
+    if (hostKeyAlgo == SshCapabilities::PubKeyDss || hostKeyAlgo == SshCapabilities::PubKeyRsa) {
+        // DSS: p and q, RSA: e and n
+        replyData.hostKeyParameters << SshPacketParser::asBigInt(input, &offset);
+        replyData.hostKeyParameters << SshPacketParser::asBigInt(input, &offset);
+
+        // g and y
+        if (hostKeyAlgo == SshCapabilities::PubKeyDss) {
+            replyData.hostKeyParameters << SshPacketParser::asBigInt(input, &offset);
+            replyData.hostKeyParameters << SshPacketParser::asBigInt(input, &offset);
+        }
+    } else {
+        QSSH_ASSERT_AND_RETURN(hostKeyAlgo == SshCapabilities::PubKeyEcdsa256);
+        if (SshPacketParser::asString(input, &offset)
+                != hostKeyAlgo.mid(11)) { // Without "ecdsa-sha2-" prefix.
+            throw SshPacketParseException();
+        }
+        replyData.q = SshPacketParser::asString(input, &offset);
+    }
+}
+
+static QByteArray &padToWidth(QByteArray &data, int targetWidth)
+{
+    return data.prepend(QByteArray(targetWidth - data.count(), 0));
+}
+
+SshKeyExchangeReply SshIncomingPacket::extractKeyExchangeReply(const QByteArray &kexAlgo,
+                                                               const QByteArray &hostKeyAlgo) const
 {
     Q_ASSERT(isComplete());
     Q_ASSERT(type() == SSH_MSG_KEXDH_REPLY);
 
     try {
         SshKeyExchangeReply replyData;
-        quint32 offset = TypeOffset + 1;
-        const quint32 k_sLength
-            = SshPacketParser::asUint32(m_data, &offset);
-        if (offset + k_sLength > currentDataSize())
-            throw SshPacketParseException();
-        replyData.k_s = m_data.mid(offset - 4, k_sLength + 4);
-        if (SshPacketParser::asString(m_data, &offset) != pubKeyAlgo)
+        quint32 topLevelOffset = TypeOffset + 1;
+        replyData.k_s = SshPacketParser::asString(m_data, &topLevelOffset);
+        quint32 k_sOffset = 0;
+        if (SshPacketParser::asString(replyData.k_s, &k_sOffset) != hostKeyAlgo)
             throw SshPacketParseException();
+        getHostKeySpecificReplyData(replyData, hostKeyAlgo, replyData.k_s.mid(k_sOffset));
 
-        // DSS: p and q, RSA: e and n
-        replyData.parameters << SshPacketParser::asBigInt(m_data, &offset);
-        replyData.parameters << SshPacketParser::asBigInt(m_data, &offset);
-
-        // g and y
-        if (pubKeyAlgo == SshCapabilities::PubKeyDss) {
-            replyData.parameters << SshPacketParser::asBigInt(m_data, &offset);
-            replyData.parameters << SshPacketParser::asBigInt(m_data, &offset);
+        if (kexAlgo == SshCapabilities::DiffieHellmanGroup1Sha1) {
+            replyData.f = SshPacketParser::asBigInt(m_data, &topLevelOffset);
+        } else {
+            QSSH_ASSERT_AND_RETURN_VALUE(kexAlgo.startsWith(SshCapabilities::EcdhKexNamePrefix),
+                                         SshKeyExchangeReply());
+            replyData.q_s = SshPacketParser::asString(m_data, &topLevelOffset);
         }
-
-        replyData.f = SshPacketParser::asBigInt(m_data, &offset);
-        offset += 4;
-        if (SshPacketParser::asString(m_data, &offset) != pubKeyAlgo)
+        const QByteArray fullSignature = SshPacketParser::asString(m_data, &topLevelOffset);
+        quint32 sigOffset = 0;
+        if (SshPacketParser::asString(fullSignature, &sigOffset) != hostKeyAlgo)
             throw SshPacketParseException();
-        replyData.signatureBlob = SshPacketParser::asString(m_data, &offset);
+        replyData.signatureBlob = SshPacketParser::asString(fullSignature, &sigOffset);
+        if (hostKeyAlgo == SshCapabilities::PubKeyEcdsa256) {
+            // Botan's PK_Verifier wants the signature in this format.
+            quint32 blobOffset = 0;
+            const Botan::BigInt r = SshPacketParser::asBigInt(replyData.signatureBlob, &blobOffset);
+            const Botan::BigInt s = SshPacketParser::asBigInt(replyData.signatureBlob, &blobOffset);
+            const int width = SshCapabilities::ecdsaIntegerWidthInBytes(hostKeyAlgo);
+            QByteArray encodedR = convertByteArray(Botan::BigInt::encode(r));
+            replyData.signatureBlob = padToWidth(encodedR, width);
+            QByteArray encodedS = convertByteArray(Botan::BigInt::encode(s));
+            replyData.signatureBlob += padToWidth(encodedS, width);
+        }
+        replyData.k_s.prepend(m_data.mid(TypeOffset + 1, 4));
         return replyData;
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_KEY_EXCHANGE_FAILED,
             "Key exchange failed: "
-            "Server sent invalid SSH_MSG_KEXDH_REPLY packet.");
+            "Server sent invalid key exchange reply packet.");
     }
 }
 
@@ -218,7 +257,7 @@ SshDisconnect SshIncomingPacket::extractDisconnect() const
         msg.reasonCode = SshPacketParser::asUint32(m_data, &offset);
         msg.description = SshPacketParser::asUserString(m_data, &offset);
         msg.language = SshPacketParser::asString(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_DISCONNECT.");
     }
@@ -237,7 +276,7 @@ SshUserAuthBanner SshIncomingPacket::extractUserAuthBanner() const
         msg.message = SshPacketParser::asUserString(m_data, &offset);
         msg.language = SshPacketParser::asString(m_data, &offset);
         return msg;
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_USERAUTH_BANNER.");
     }
@@ -262,7 +301,7 @@ SshUserAuthInfoRequestPacket SshIncomingPacket::extractUserAuthInfoRequest() con
             msg.echos << SshPacketParser::asBool(m_data, &offset);
         }
         return msg;
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_USERAUTH_INFO_REQUEST.");
     }
@@ -280,7 +319,7 @@ SshDebug SshIncomingPacket::extractDebug() const
         msg.message = SshPacketParser::asUserString(m_data, &offset);
         msg.language = SshPacketParser::asString(m_data, &offset);
         return msg;
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_DEBUG.");
     }
@@ -296,7 +335,7 @@ SshUnimplemented SshIncomingPacket::extractUnimplemented() const
         quint32 offset = TypeOffset + 1;
         msg.invalidMsgSeqNr = SshPacketParser::asUint32(m_data, &offset);
         return msg;
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_UNIMPLEMENTED.");
     }
@@ -314,7 +353,7 @@ SshChannelOpenFailure SshIncomingPacket::extractChannelOpenFailure() const
         openFailure.reasonCode = SshPacketParser::asUint32(m_data, &offset);
         openFailure.reasonString = QString::fromLocal8Bit(SshPacketParser::asString(m_data, &offset));
         openFailure.language = SshPacketParser::asString(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Server sent invalid SSH_MSG_CHANNEL_OPEN_FAILURE packet.");
     }
@@ -333,7 +372,7 @@ SshChannelOpenConfirmation SshIncomingPacket::extractChannelOpenConfirmation() c
         confirmation.remoteChannel = SshPacketParser::asUint32(m_data, &offset);
         confirmation.remoteWindowSize = SshPacketParser::asUint32(m_data, &offset);
         confirmation.remoteMaxPacketSize = SshPacketParser::asUint32(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Server sent invalid SSH_MSG_CHANNEL_OPEN_CONFIRMATION packet.");
     }
@@ -350,7 +389,7 @@ SshChannelWindowAdjust SshIncomingPacket::extractWindowAdjust() const
         quint32 offset = TypeOffset + 1;
         adjust.localChannel = SshPacketParser::asUint32(m_data, &offset);
         adjust.bytesToAdd = SshPacketParser::asUint32(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_CHANNEL_WINDOW_ADJUST packet.");
     }
@@ -367,7 +406,7 @@ SshChannelData SshIncomingPacket::extractChannelData() const
         quint32 offset = TypeOffset + 1;
         data.localChannel = SshPacketParser::asUint32(m_data, &offset);
         data.data = SshPacketParser::asString(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_CHANNEL_DATA packet.");
     }
@@ -385,7 +424,7 @@ SshChannelExtendedData SshIncomingPacket::extractChannelExtendedData() const
         data.localChannel = SshPacketParser::asUint32(m_data, &offset);
         data.type = SshPacketParser::asUint32(m_data, &offset);
         data.data = SshPacketParser::asString(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_CHANNEL_EXTENDED_DATA packet.");
     }
@@ -407,7 +446,7 @@ SshChannelExitStatus SshIncomingPacket::extractChannelExitStatus() const
         if (SshPacketParser::asBool(m_data, &offset))
             throw SshPacketParseException();
         exitStatus.exitStatus = SshPacketParser::asUint32(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid exit-status packet.");
     }
@@ -432,7 +471,7 @@ SshChannelExitSignal SshIncomingPacket::extractChannelExitSignal() const
         exitSignal.coreDumped = SshPacketParser::asBool(m_data, &offset);
         exitSignal.error = SshPacketParser::asUserString(m_data, &offset);
         exitSignal.language = SshPacketParser::asString(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid exit-signal packet.");
     }
@@ -446,7 +485,7 @@ quint32 SshIncomingPacket::extractRecipientChannel() const
     try {
         quint32 offset = TypeOffset + 1;
         return SshPacketParser::asUint32(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Server sent invalid packet.");
     }
@@ -461,7 +500,7 @@ QByteArray SshIncomingPacket::extractChannelRequestType() const
         quint32 offset = TypeOffset + 1;
         SshPacketParser::asUint32(m_data, &offset);
         return SshPacketParser::asString(m_data, &offset);
-    } catch (SshPacketParseException &) {
+    } catch (const SshPacketParseException &) {
         throw SSH_SERVER_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
             "Invalid SSH_MSG_CHANNEL_REQUEST packet.");
     }
