@@ -33,6 +33,10 @@
 
 #include "debugger_global.h"
 #include "debuggerconstants.h"
+#include "debuggerstartparameters.h"
+
+#include <projectexplorer/devicesupport/idevice.h>
+#include <texteditor/textmark.h>
 
 #include <QObject>
 #include <QProcess>
@@ -43,16 +47,13 @@ class QPoint;
 class QAbstractItemModel;
 QT_END_NAMESPACE
 
-namespace TextEditor { class TextEditorWidget; }
 namespace Core { class IOptionsPage; }
 
 namespace Debugger {
 
 class DebuggerRunControl;
-class DebuggerStartParameters;
 class RemoteSetupResult;
 
-DEBUGGER_EXPORT QDebug operator<<(QDebug str, const DebuggerStartParameters &);
 DEBUGGER_EXPORT QDebug operator<<(QDebug str, DebuggerState state);
 
 namespace Internal {
@@ -65,6 +66,7 @@ class MemoryAgent;
 class WatchData;
 class WatchItem;
 class BreakHandler;
+class LocationMark;
 class ModulesHandler;
 class RegisterHandler;
 class StackHandler;
@@ -80,6 +82,57 @@ class MemoryViewSetupData;
 class Terminal;
 class ThreadId;
 
+class DebuggerRunParameters : public DebuggerStartParameters
+{
+public:
+    DebuggerRunParameters() {}
+
+    DebuggerEngineType masterEngineType = NoEngineType;
+    DebuggerEngineType cppEngineType = NoEngineType;
+
+    DebuggerLanguages languages = AnyLanguage;
+    bool breakOnMain = false;
+    bool multiProcess = false; // Whether to set detach-on-fork off.
+
+    QString debuggerCommand;
+    QString coreFile;
+    QString overrideStartScript; // Used in attach to core and remote debugging
+    QString startMessage; // First status message shown.
+    QMap<QString, QString> sourcePathMap;
+    QString debugInfoLocation; // Gdb "set-debug-file-directory".
+    QStringList debugSourceLocation; // Gdb "directory"
+    QString serverStartScript;
+    ProjectExplorer::IDevice::ConstPtr device;
+    QString sysRoot;
+    bool isSnapshot = false; // Set if created internally.
+    ProjectExplorer::Abi toolChainAbi;
+
+    QString projectSourceDirectory;
+    QStringList projectSourceFiles;
+
+    // Used by AttachCrashedExternal.
+    QString crashParameter;
+
+    // For Debugger testing.
+    int testCase = 0;
+};
+
+class UpdateParameters
+{
+public:
+    UpdateParameters() {}
+
+    QList<QByteArray> partialVariables() const
+    {
+        QList<QByteArray> result;
+        if (!partialVariable.isEmpty())
+            result.append(partialVariable);
+        return result;
+    }
+
+    QByteArray partialVariable;
+};
+
 class Location
 {
 public:
@@ -117,15 +170,18 @@ private:
     quint64 m_address;
 };
 
+enum LocationType { UnknownLocation, LocationByFile, LocationByAddress };
+
 class ContextData
 {
 public:
-    ContextData() : lineNumber(0), address(0) {}
+    bool isValid() const { return type != UnknownLocation; }
 
 public:
+    LocationType type = UnknownLocation;
     QString fileName;
-    int lineNumber;
-    quint64 address;
+    int lineNumber = 0;
+    quint64 address = 0;
 };
 
 class DebuggerEngine : public QObject
@@ -133,16 +189,16 @@ class DebuggerEngine : public QObject
     Q_OBJECT
 
 public:
-    explicit DebuggerEngine(const DebuggerStartParameters &sp);
+    explicit DebuggerEngine(const DebuggerRunParameters &sp);
     virtual ~DebuggerEngine();
 
-    const DebuggerStartParameters &startParameters() const;
-    DebuggerStartParameters &startParameters();
+    const DebuggerRunParameters &runParameters() const;
+    DebuggerRunParameters &runParameters();
 
-    virtual bool setToolTipExpression(const Internal::DebuggerToolTipContext &);
-
-    virtual void updateWatchItem(WatchItem *) {}
-    virtual void watchDataSelected(const QByteArray &iname);
+    virtual bool canHandleToolTip(const DebuggerToolTipContext &) const;
+    virtual void expandItem(const QByteArray &iname); // Called when item in tree gets expanded.
+    virtual void updateItem(const QByteArray &iname); // Called for fresh watch items.
+    virtual void selectWatchData(const QByteArray &iname);
 
     virtual void startDebugger(DebuggerRunControl *runControl);
 
@@ -190,6 +246,7 @@ public:
 
     virtual void createSnapshot();
     virtual void updateAll();
+    virtual void updateLocals();
 
     virtual bool stateAcceptsBreakpointChanges() const { return true; }
     virtual void attemptBreakpointSynchronization();
@@ -254,11 +311,12 @@ public:
     DebuggerEngine *masterEngine() const;
     virtual DebuggerEngine *cppEngine() { return 0; }
 
-    virtual bool canDisplayTooltip() const { return state() == InferiorStopOk; }
+    virtual bool canDisplayTooltip() const;
 
     virtual void notifyInferiorIll();
 
     QString toFileInProject(const QUrl &fileUrl);
+    void updateBreakpointMarkers();
 
 signals:
     void stateChanged(Debugger::DebuggerState state);
@@ -349,6 +407,8 @@ protected:
     virtual void frameUp();
     virtual void frameDown();
 
+    virtual void doUpdateLocals(const UpdateParameters &params);
+
     void setTargetState(DebuggerState state);
     void setMasterEngine(DebuggerEngine *masterEngine);
 
@@ -366,7 +426,7 @@ protected:
     bool isStateDebugging() const;
     void setStateDebugging(bool on);
 
-    static void validateExecutable(DebuggerStartParameters *sp);
+    static void validateExecutable(DebuggerRunParameters *sp);
 
     virtual void setupSlaveInferior();
     virtual void setupSlaveEngine();
@@ -387,12 +447,30 @@ private:
     virtual void setState(DebuggerState state, bool forced = false);
 
     friend class DebuggerEnginePrivate;
+    friend class LocationMark;
     DebuggerEnginePrivate *d;
 };
 
+class LocationMark : public TextEditor::TextMark
+{
+public:
+    LocationMark(DebuggerEngine *engine, const QString &file, int line);
+
+private:
+    bool isDraggable() const;
+    void dragToLine(int line);
+
+    QPointer<DebuggerEngine> m_engine;
+};
+
+DebuggerEngine *createEngine(DebuggerEngineType et, const DebuggerRunParameters &rp, QStringList *errors);
+
+DebuggerRunControl *createAndScheduleRun(const DebuggerRunParameters &rp, const ProjectExplorer::Kit *kit);
+
 } // namespace Internal
 } // namespace Debugger
 
+Q_DECLARE_METATYPE(Debugger::Internal::UpdateParameters)
 Q_DECLARE_METATYPE(Debugger::Internal::ContextData)
 
 #endif // DEBUGGER_DEBUGGERENGINE_H
